from aws_cdk import Stack
from aws_cdk import aws_lambda as _lambda
from aws_cdk import aws_ecr as _ecr
from aws_cdk import aws_events as _events
from aws_cdk import aws_events_targets as _events_targets
from aws_cdk import aws_iam as _iam
from aws_cdk import Aws, Duration
from constructs import Construct


class ExchangeRatesLambdaStack(Stack):
    def __init__(
        self, scope: Construct, construct_id: str, image_name: str, **kwargs
    ) -> None:
        super().__init__(scope, construct_id, **kwargs)
        self.build_lambda_func(self.get_lambda_image(image_name))

    def get_lambda_image(self, image_name: str):
        ecr_repository = _ecr.Repository.from_repository_attributes(
            self,
            id="ECR",
            repository_arn="arn:aws:ecr:{0}:{1}:repository".format(
                Aws.REGION, Aws.ACCOUNT_ID
            ),
            repository_name=image_name,
        )
        ecr_image = _lambda.DockerImageCode.from_ecr(
            repository=ecr_repository,
            tag="latest",
            cmd=[
                "budgetguard.core.lambda_functions.master_exchange_rates.lambda_handler"  # noqa
            ],  # noqa
            entrypoint=["python", "-m", "awslambdaric"],
        )
        return ecr_image

    def build_lambda_func(self, lambda_image: _lambda.Code):
        exchange_rates_lambda = _lambda.DockerImageFunction(
            scope=self,
            id="ExchangeRatesLambda",
            function_name="ExchangeRatesLambda",
            code=lambda_image,
            timeout=Duration.seconds(300),
            memory_size=1024,
        )
        exchange_rates_lambda.add_to_role_policy(
            _iam.PolicyStatement(
                actions=["s3:GetObject", "s3:PutObject"],
                resources=["arn:aws:s3:::budget-guard-master/*"],
                effect=_iam.Effect.ALLOW,
            )
        )

        exchange_rates_lambda.add_to_role_policy(
            _iam.PolicyStatement(
                actions=["secretsmanager:GetSecretValue"],
                resources=[
                    "arn:aws:secretsmanager:us-east-1:327077392103:secret:budget_guard_nordigen_key-OCfS6T"  # noqa
                ],
                effect=_iam.Effect.ALLOW,
            )
        )
        # Rule to trigger the lambda every day at 1 AM
        rule = _events.Rule(
            self,
            id="ExchangeRatesRule",
            rule_name="ExchangeRatesRule",
            schedule=_events.Schedule.cron(
                minute="0", hour="1", month="*", week_day="*", year="*"
            ),
        )
        rule.add_target(_events_targets.LambdaFunction(exchange_rates_lambda))



import argparse
import sys
import os
from .core.pipelines.pipeline import Pipeline
from dotenv import load_dotenv

here = os.path.dirname(__file__)

sys.path.append(os.path.join(here, ".."))


parser = argparse.ArgumentParser()

parser.add_argument(
    "-pid",
    "--partition-id",
    help="The partition of the datalake to read from.",
    type=str,
    required=True,
)

parser.add_argument(
    "-t",
    "--task",
    help="The task to run.",
    type=str,
    required=True,
)


def run_task(pipeline: Pipeline):
    pipeline.run()


def run(task: str, partition_id: str):
    load_dotenv()
    if task == "ingest_account_data":
        from .core.pipelines.ingest_account_data import (
            IngestAccountData,
        )

        pipeline = IngestAccountData(partition_id)
    elif task == "master_exchange_rates":
        from .core.pipelines.master.exchange_rates_pipelines import (
            ExchangeRatesPipeline,
        )

        pipeline = ExchangeRatesPipeline(partition_id)
    elif task == "bronze_to_silver_balances":
        from .core.pipelines.bronze_to_silver.balances_pipeline import (
            BronzeToSilverBalancesPipeline,
        )

        pipeline = BronzeToSilverBalancesPipeline(partition_id)
    elif task == "bronze_to_silver_details":
        from .core.pipelines.bronze_to_silver.details_pipeline import (
            BronzeToSilverDetailsPipeline,
        )

        pipeline = BronzeToSilverDetailsPipeline(partition_id)
    elif task == "bronze_to_silver_transactions":
        from .core.pipelines.bronze_to_silver.transactions_pipeline import (
            BronzeToSilverTransactionsPipeline,
        )

        pipeline = BronzeToSilverTransactionsPipeline(partition_id)
    elif task == "bronze_to_silver_metadata":
        from .core.pipelines.bronze_to_silver.metadata_pipeline import (
            BronzeToSilverMetadataPipeline,
        )

        pipeline = BronzeToSilverMetadataPipeline(partition_id)
    else:
        raise ValueError(f"Unknown task: {task}")
    run_task(pipeline=pipeline)


if __name__ == "__main__":
    args = parser.parse_args()
    run(args.task, args.partition_id)



import sys
import os
import json

here = os.path.dirname(__file__)

sys.path.append(os.path.join(here, ".."))

from .pipeline import Pipeline  # noqa: E402
from datalake import Datalake  # noqa: E402
from data_access_service.data_loaders import (  # noqa: E402
    create_data_loader,
)
from loguru import logger  # noqa: E402


class IngestAccountData(Pipeline):
    INPUT_DATA_LOADER = "nordigen"
    OUTPUT_DATA_LOADER = "s3"
    OUTPUT_LAYER = "ingest"
    OUTPUT_TABLES_KEYS = ["balances", "transactions", "details", "metadata"]

    def __init__(self, partition_id: str) -> None:
        self.datalake = Datalake()
        self.partition_id = partition_id
        self.input_loader = create_data_loader(self.INPUT_DATA_LOADER)
        self.output_loader = create_data_loader(self.OUTPUT_DATA_LOADER)

    def read_sources(self):
        """
        Reads the data from the data sources.
        """
        nordigen_raw_data = self.input_loader.read(self.partition_id)
        return nordigen_raw_data

    def write_sources(self, nordigen_data):
        """
        Writes the data to the data sources.
        """
        account_ids = list(nordigen_data.keys())
        for key in self.OUTPUT_TABLES_KEYS:
            datalake_config = self.datalake[self.OUTPUT_LAYER][key]
            for account_id in account_ids:
                partition_config = {
                    "partition_id": self.partition_id,
                    "account_id": account_id,
                }
                data = nordigen_data[account_id][key]
                self.output_loader.write(
                    json.dumps(data), datalake_config, partition_config
                )

    def transform(self):
        """
        Transforms the data.
        """
        raise NotImplementedError(
            "Transform method not needed for ingestion pipeline!"
        )

    def run(self):
        """
        Runs the pipeline.
        """
        logger.info("Running the ingestion pipeline...")
        nordigen_raw_data = self.read_sources()
        logger.info("Writing the data to the datalake...")
        self.write_sources(nordigen_raw_data)
        logger.info("Finished running the ingestion pipeline!")



from aws_cdk import Stack, Aws
from aws_cdk import aws_glue as glue
from constructs import Construct


class GlueDataCatalogStack(Stack):
    def __init__(self, scope: Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a Ingest Glue Data Catalog database
        glue.CfnDatabase(
            self,
            "budget-guard-ingest-database",
            catalog_id=Aws.ACCOUNT_ID,
            database_input={
                "name": "budget-guard-ingest-database",
                "description": "AWS Glue Database for ingestion",
            },
        )

        # Create a Bronze Glue Data Catalog database
        glue.CfnDatabase(
            self,
            "budget-guard-bronze-database",
            catalog_id=Aws.ACCOUNT_ID,
            database_input={
                "name": "budget-guard-bronze-database",
                "description": "AWS Glue Database for bronze",
            },
        )


class GlueCrawlersStack(Stack):
    def __init__(self, scope: Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a Ingestion Glue Crawler
        glue.CfnCrawler(
            self,
            "budget-guard-ingest-crawler",
            name="budget-guard-ingest-crawler",
            role=f"arn:aws:iam::{Aws.ACCOUNT_ID}:role/service-role/AWSGlueServiceRole",  # noqa
            database_name="budget-guard-ingest-database",
            targets={"s3Targets": [{"path": "s3://budget-guard-ingest"}]},
            # schedule=glue.CfnCrawler.ScheduleProperty(schedule_expression="cron(0 1 * * ? *)"),  # noqa
        )

        # Create a Bronze Glue Crawler
        glue.CfnCrawler(
            self,
            "budget-guard-bronze-crawler",
            name="budget-guard-bronze-crawler",
            role=f"arn:aws:iam::{Aws.ACCOUNT_ID}:role/service-role/AWSGlueServiceRole",  # noqa
            database_name="budget-guard-bronze-database",
            targets={"s3Targets": [{"path": "s3://budget-guard-bronze"}]},
            # schedule=glue.CfnCrawler.ScheduleProperty(schedule_expression="cron(0 1 * * ? *)"),  # noqa
        )



from abc import ABC, abstractmethod
import requests
from loguru import logger
import os
from enum import Enum


class Connection(ABC):
    class __Platform__(Enum):
        LOCAL = "local"
        EMR = "emr"

    def __init__(self) -> None:
        """
        Constructor for Pipeline class.
        """
        self.platform = self.__set_platform__()

    def __set_platform__(self) -> str:
        """
        Sets the platform.
        """
        if "EMR_CLUSTER_ID" in os.environ:
            return self.__Platform__.EMR
        else:
            return self.__Platform__.LOCAL

    @abstractmethod
    def connect(self):
        """
        Base method for connecting to the data source.
        """
        raise NotImplementedError()

    def request(
        self,
        url: str,
        method: str,
        params: dict = None,
        data: dict = None,
        headers: dict = None,
    ) -> requests.Response:
        """
        Method to make a request to a given url.

        :param url: The url to make the request to.
        :param method: The HTTP method to use.
        :param params: The parameters to send with the request.
        :param data: The data to send
        :param headers: The headers to send with the request.

        :return: The response object.
        """
        logger.info(f"Making {method} request to {url}...")
        try:
            response = requests.request(
                method=method,
                url=url,
                params=params,
                data=data,
                headers=headers,
            )
            response.raise_for_status()
            return response
        except requests.exceptions.HTTPError as e:
            logger.error(f"Error making request to {url}: {e}")
            raise e



from budgetguard.core.pipelines.pipeline import Pipeline
import sys


def run_task(pipeline: Pipeline):
    pipeline.run()


def run(task: str, partition_id: str):
    if task == "ingest_account_data":
        from budgetguard.core.pipelines.ingest_account_data import (
            IngestAccountData,
        )

        pipeline = IngestAccountData(partition_id)
    elif task == "bronze_to_silver_balances":
        from budgetguard.core.pipelines.bronze_to_silver.balances_pipeline import (  # noqa: E501
            BronzeToSilverBalancesPipeline,
        )

        pipeline = BronzeToSilverBalancesPipeline(partition_id)
    elif task == "bronze_to_silver_details":
        from budgetguard.core.pipelines.bronze_to_silver.details_pipeline import (  # noqa: E501
            BronzeToSilverDetailsPipeline,
        )

        pipeline = BronzeToSilverDetailsPipeline(partition_id)
    elif task == "bronze_to_silver_transactions":
        from budgetguard.core.pipelines.bronze_to_silver.transactions_pipeline import (  # noqa: E501
            BronzeToSilverTransactionsPipeline,
        )

        pipeline = BronzeToSilverTransactionsPipeline(partition_id)
    elif task == "bronze_to_silver_metadata":
        from budgetguard.core.pipelines.bronze_to_silver.metadata_pipeline import (  # noqa: E501
            BronzeToSilverMetadataPipeline,
        )

        pipeline = BronzeToSilverMetadataPipeline(partition_id)
    else:
        raise ValueError(f"Unknown task: {task}")
    run_task(pipeline)


if __name__ == "__main__":
    run(sys.argv[1], sys.argv[2])



from .connection import Connection
from forex_python.converter import CurrencyRates


class ExchangeRatesAPIConnection(Connection):
    def __init__(self) -> None:
        super().__init__()
        self.connection: CurrencyRates = self.connect()

    def connect(self):
        return CurrencyRates()
